/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package com.totsp.gwt.glassfish;

import java.awt.image.BufferedImage;
import java.awt.image.DataBufferByte;
import java.io.File;
import java.io.IOException;
import java.net.BindException;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.imageio.ImageIO;

import org.glassfish.api.deployment.DeployCommandParameters;
import org.glassfish.internal.embedded.ContainerBuilder;
import org.glassfish.internal.embedded.EmbeddedDeployer;
import org.glassfish.internal.embedded.EmbeddedFileSystem;
import org.glassfish.internal.embedded.LifecycleException;
import org.glassfish.internal.embedded.Server;

import com.google.gwt.core.ext.ServletContainer;
import com.google.gwt.core.ext.ServletContainerLauncher;
import com.google.gwt.core.ext.TreeLogger;
import com.google.gwt.core.ext.TreeLogger.Type;
import com.google.gwt.core.ext.UnableToCompleteException;


/**
 *
 * @author robert.cooper
 */
public class Glassfish extends ServletContainerLauncher {
    EmbeddedDeployer deployer;
    Server server;
    byte[] icon;
    volatile boolean running = true;

    public Glassfish() {
        try {
            BufferedImage img = ImageIO.read(Glassfish.class.getResourceAsStream("/gf-icon.png"));
            DataBufferByte buffer = (DataBufferByte) img.getRaster()
                                                        .getDataBuffer();
            this.icon = buffer.getData();
            System.out.println(icon+" "+icon.length);
        } catch (IOException ex) {
            Logger.getLogger(Glassfish.class.getName())
                  .log(Level.SEVERE, null, ex);
            throw new RuntimeException(ex);
        }
    }

    @Override
    public byte[] getIconBytes() {
        return this.icon;
    }

    @Override
    public String getName() {
        return "GlassFish";
    }

    public ServletContainer start(final TreeLogger logger, final int port, final File appRootDir)
        throws BindException, Exception {
        Server.Builder builder = new Server.Builder("GWT");
        builder.verbose(true);
        File installDir = new File("C:\\glassfish3\\glassfish");
        File domainDir = new File(installDir, "domains\\domain1");
        File domainConfig = new File(domainDir, "config");
        File domainXml = new File(domainConfig, "domain.xml");
        EmbeddedFileSystem.Builder efsb = new EmbeddedFileSystem.Builder();
        efsb.installRoot(installDir);
        efsb.instanceRoot(domainDir);
        efsb.configurationFile(domainXml);
        EmbeddedFileSystem efs = efsb.build();
        builder.embeddedFileSystem(efs);
        
        server = builder.build();
        server.createPort(port);
//        ContainerBuilder containerBuilder = server.createConfig(ContainerBuilder.Type.web);
//        server.addContainer(containerBuilder);
//        containerBuilder.create(server); 
        server.start();
        deployer = server.getDeployer();
        final DeployCommandParameters params = new DeployCommandParameters();
        params.name = "root";
        params.contextroot = "/";
        System.out.println("Deploying "+appRootDir);
        deployer.deploy(appRootDir, params);
        System.out.println("Deployed");
        logger.log(Type.WARN, "See console log for server log.");

        return new ServletContainer() {
                @Override
                public int getPort() {
                    return port;
                }

                @Override
                public void refresh() throws UnableToCompleteException {
                    deployer.undeployAll();
                    deployer.deploy(appRootDir, params);
                }

                @Override
                public void stop() throws UnableToCompleteException {
                    deployer.undeployAll();

                    try {
                        running = false;
                        server.stop();
                    } catch (LifecycleException ex) {
                        logger.log(Type.ERROR, "Exception shutting down glassfish", ex);
                        Logger.getLogger(Glassfish.class.getName())
                              .log(Level.SEVERE, null, ex);
                        throw new UnableToCompleteException();
                    }
                }
            };
    }
}
